# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rastervision/protos/tf_object_detection/box_predictor.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from rastervision.protos.tf_object_detection import hyperparams_pb2 as rastervision_dot_protos_dot_tf__object__detection_dot_hyperparams__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='rastervision/protos/tf_object_detection/box_predictor.proto',
  package='rastervision.protos.tf_object_detection',
  syntax='proto2',
  serialized_pb=_b('\n;rastervision/protos/tf_object_detection/box_predictor.proto\x12\'rastervision.protos.tf_object_detection\x1a\x39rastervision/protos/tf_object_detection/hyperparams.proto\"\xd1\x03\n\x0c\x42oxPredictor\x12i\n\x1b\x63onvolutional_box_predictor\x18\x01 \x01(\x0b\x32\x42.rastervision.protos.tf_object_detection.ConvolutionalBoxPredictorH\x00\x12`\n\x17mask_rcnn_box_predictor\x18\x02 \x01(\x0b\x32=.rastervision.protos.tf_object_detection.MaskRCNNBoxPredictorH\x00\x12W\n\x12rfcn_box_predictor\x18\x03 \x01(\x0b\x32\x39.rastervision.protos.tf_object_detection.RfcnBoxPredictorH\x00\x12\x83\x01\n)weight_shared_convolutional_box_predictor\x18\x04 \x01(\x0b\x32N.rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictorH\x00\x42\x15\n\x13\x62ox_predictor_oneof\"c\n\x08MaskHead\x12\x17\n\x0bmask_height\x18\x01 \x01(\x05:\x02\x31\x35\x12\x16\n\nmask_width\x18\x02 \x01(\x05:\x02\x31\x35\x12&\n\x18masks_are_class_agnostic\x18\x03 \x01(\x08:\x04true\"\xe6\x03\n\x19\x43onvolutionalBoxPredictor\x12N\n\x10\x63onv_hyperparams\x18\x01 \x01(\x0b\x32\x34.rastervision.protos.tf_object_detection.Hyperparams\x12\x14\n\tmin_depth\x18\x02 \x01(\x05:\x01\x30\x12\x14\n\tmax_depth\x18\x03 \x01(\x05:\x01\x30\x12&\n\x1bnum_layers_before_predictor\x18\x04 \x01(\x05:\x01\x30\x12\x19\n\x0buse_dropout\x18\x05 \x01(\x08:\x04true\x12%\n\x18\x64ropout_keep_probability\x18\x06 \x01(\x02:\x03\x30.8\x12\x16\n\x0bkernel_size\x18\x07 \x01(\x05:\x01\x31\x12\x18\n\rbox_code_size\x18\x08 \x01(\x05:\x01\x34\x12&\n\x17\x61pply_sigmoid_to_scores\x18\t \x01(\x08:\x05\x66\x61lse\x12%\n\x1a\x63lass_prediction_bias_init\x18\n \x01(\x02:\x01\x30\x12\x1c\n\ruse_depthwise\x18\x0b \x01(\x08:\x05\x66\x61lse\x12\x44\n\tmask_head\x18\x0c \x01(\x0b\x32\x31.rastervision.protos.tf_object_detection.MaskHead\"\xc4\x06\n%WeightSharedConvolutionalBoxPredictor\x12N\n\x10\x63onv_hyperparams\x18\x01 \x01(\x0b\x32\x34.rastervision.protos.tf_object_detection.Hyperparams\x12&\n\x1bnum_layers_before_predictor\x18\x04 \x01(\x05:\x01\x30\x12\x10\n\x05\x64\x65pth\x18\x02 \x01(\x05:\x01\x30\x12\x16\n\x0bkernel_size\x18\x07 \x01(\x05:\x01\x33\x12\x18\n\rbox_code_size\x18\x08 \x01(\x05:\x01\x34\x12%\n\x1a\x63lass_prediction_bias_init\x18\n \x01(\x02:\x01\x30\x12\x1a\n\x0buse_dropout\x18\x0b \x01(\x08:\x05\x66\x61lse\x12%\n\x18\x64ropout_keep_probability\x18\x0c \x01(\x02:\x03\x30.8\x12%\n\x16share_prediction_tower\x18\r \x01(\x08:\x05\x66\x61lse\x12\x1c\n\ruse_depthwise\x18\x0e \x01(\x08:\x05\x66\x61lse\x12\x44\n\tmask_head\x18\x0f \x01(\x0b\x32\x31.rastervision.protos.tf_object_detection.MaskHead\x12\x80\x01\n\x0fscore_converter\x18\x10 \x01(\x0e\x32].rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.ScoreConverter:\x08IDENTITY\x12\x86\x01\n\x18\x62ox_encodings_clip_range\x18\x11 \x01(\x0b\x32\x64.rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.BoxEncodingsClipRange\x1a\x31\n\x15\x42oxEncodingsClipRange\x12\x0b\n\x03min\x18\x01 \x01(\x02\x12\x0b\n\x03max\x18\x02 \x01(\x02\"+\n\x0eScoreConverter\x12\x0c\n\x08IDENTITY\x10\x00\x12\x0b\n\x07SIGMOID\x10\x01\"\xb2\x04\n\x14MaskRCNNBoxPredictor\x12L\n\x0e\x66\x63_hyperparams\x18\x01 \x01(\x0b\x32\x34.rastervision.protos.tf_object_detection.Hyperparams\x12\x1a\n\x0buse_dropout\x18\x02 \x01(\x08:\x05\x66\x61lse\x12%\n\x18\x64ropout_keep_probability\x18\x03 \x01(\x02:\x03\x30.5\x12\x18\n\rbox_code_size\x18\x04 \x01(\x05:\x01\x34\x12N\n\x10\x63onv_hyperparams\x18\x05 \x01(\x0b\x32\x34.rastervision.protos.tf_object_detection.Hyperparams\x12%\n\x16predict_instance_masks\x18\x06 \x01(\x08:\x05\x66\x61lse\x12\'\n\x1amask_prediction_conv_depth\x18\x07 \x01(\x05:\x03\x32\x35\x36\x12 \n\x11predict_keypoints\x18\x08 \x01(\x08:\x05\x66\x61lse\x12\x17\n\x0bmask_height\x18\t \x01(\x05:\x02\x31\x35\x12\x16\n\nmask_width\x18\n \x01(\x05:\x02\x31\x35\x12*\n\x1fmask_prediction_num_conv_layers\x18\x0b \x01(\x05:\x01\x32\x12\'\n\x18masks_are_class_agnostic\x18\x0c \x01(\x08:\x05\x66\x61lse\x12\'\n\x18share_box_across_classes\x18\r \x01(\x08:\x05\x66\x61lse\"\x89\x02\n\x10RfcnBoxPredictor\x12N\n\x10\x63onv_hyperparams\x18\x01 \x01(\x0b\x32\x34.rastervision.protos.tf_object_detection.Hyperparams\x12\"\n\x17num_spatial_bins_height\x18\x02 \x01(\x05:\x01\x33\x12!\n\x16num_spatial_bins_width\x18\x03 \x01(\x05:\x01\x33\x12\x13\n\x05\x64\x65pth\x18\x04 \x01(\x05:\x04\x31\x30\x32\x34\x12\x18\n\rbox_code_size\x18\x05 \x01(\x05:\x01\x34\x12\x17\n\x0b\x63rop_height\x18\x06 \x01(\x05:\x02\x31\x32\x12\x16\n\ncrop_width\x18\x07 \x01(\x05:\x02\x31\x32')
  ,
  dependencies=[rastervision_dot_protos_dot_tf__object__detection_dot_hyperparams__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR_SCORECONVERTER = _descriptor.EnumDescriptor(
  name='ScoreConverter',
  full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.ScoreConverter',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='IDENTITY', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SIGMOID', index=1, number=1,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=2015,
  serialized_end=2058,
)
_sym_db.RegisterEnumDescriptor(_WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR_SCORECONVERTER)


_BOXPREDICTOR = _descriptor.Descriptor(
  name='BoxPredictor',
  full_name='rastervision.protos.tf_object_detection.BoxPredictor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='convolutional_box_predictor', full_name='rastervision.protos.tf_object_detection.BoxPredictor.convolutional_box_predictor', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mask_rcnn_box_predictor', full_name='rastervision.protos.tf_object_detection.BoxPredictor.mask_rcnn_box_predictor', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='rfcn_box_predictor', full_name='rastervision.protos.tf_object_detection.BoxPredictor.rfcn_box_predictor', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='weight_shared_convolutional_box_predictor', full_name='rastervision.protos.tf_object_detection.BoxPredictor.weight_shared_convolutional_box_predictor', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='box_predictor_oneof', full_name='rastervision.protos.tf_object_detection.BoxPredictor.box_predictor_oneof',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=164,
  serialized_end=629,
)


_MASKHEAD = _descriptor.Descriptor(
  name='MaskHead',
  full_name='rastervision.protos.tf_object_detection.MaskHead',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mask_height', full_name='rastervision.protos.tf_object_detection.MaskHead.mask_height', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=15,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mask_width', full_name='rastervision.protos.tf_object_detection.MaskHead.mask_width', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=15,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='masks_are_class_agnostic', full_name='rastervision.protos.tf_object_detection.MaskHead.masks_are_class_agnostic', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=True,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=631,
  serialized_end=730,
)


_CONVOLUTIONALBOXPREDICTOR = _descriptor.Descriptor(
  name='ConvolutionalBoxPredictor',
  full_name='rastervision.protos.tf_object_detection.ConvolutionalBoxPredictor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='conv_hyperparams', full_name='rastervision.protos.tf_object_detection.ConvolutionalBoxPredictor.conv_hyperparams', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='min_depth', full_name='rastervision.protos.tf_object_detection.ConvolutionalBoxPredictor.min_depth', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_depth', full_name='rastervision.protos.tf_object_detection.ConvolutionalBoxPredictor.max_depth', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='num_layers_before_predictor', full_name='rastervision.protos.tf_object_detection.ConvolutionalBoxPredictor.num_layers_before_predictor', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='use_dropout', full_name='rastervision.protos.tf_object_detection.ConvolutionalBoxPredictor.use_dropout', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=True,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dropout_keep_probability', full_name='rastervision.protos.tf_object_detection.ConvolutionalBoxPredictor.dropout_keep_probability', index=5,
      number=6, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0.8),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='kernel_size', full_name='rastervision.protos.tf_object_detection.ConvolutionalBoxPredictor.kernel_size', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='box_code_size', full_name='rastervision.protos.tf_object_detection.ConvolutionalBoxPredictor.box_code_size', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=4,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='apply_sigmoid_to_scores', full_name='rastervision.protos.tf_object_detection.ConvolutionalBoxPredictor.apply_sigmoid_to_scores', index=8,
      number=9, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='class_prediction_bias_init', full_name='rastervision.protos.tf_object_detection.ConvolutionalBoxPredictor.class_prediction_bias_init', index=9,
      number=10, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='use_depthwise', full_name='rastervision.protos.tf_object_detection.ConvolutionalBoxPredictor.use_depthwise', index=10,
      number=11, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mask_head', full_name='rastervision.protos.tf_object_detection.ConvolutionalBoxPredictor.mask_head', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=733,
  serialized_end=1219,
)


_WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR_BOXENCODINGSCLIPRANGE = _descriptor.Descriptor(
  name='BoxEncodingsClipRange',
  full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.BoxEncodingsClipRange',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='min', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.BoxEncodingsClipRange.min', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.BoxEncodingsClipRange.max', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1964,
  serialized_end=2013,
)

_WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR = _descriptor.Descriptor(
  name='WeightSharedConvolutionalBoxPredictor',
  full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='conv_hyperparams', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.conv_hyperparams', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='num_layers_before_predictor', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.num_layers_before_predictor', index=1,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='depth', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.depth', index=2,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='kernel_size', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.kernel_size', index=3,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=3,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='box_code_size', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.box_code_size', index=4,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=4,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='class_prediction_bias_init', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.class_prediction_bias_init', index=5,
      number=10, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='use_dropout', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.use_dropout', index=6,
      number=11, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dropout_keep_probability', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.dropout_keep_probability', index=7,
      number=12, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0.8),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='share_prediction_tower', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.share_prediction_tower', index=8,
      number=13, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='use_depthwise', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.use_depthwise', index=9,
      number=14, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mask_head', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.mask_head', index=10,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='score_converter', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.score_converter', index=11,
      number=16, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='box_encodings_clip_range', full_name='rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.box_encodings_clip_range', index=12,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR_BOXENCODINGSCLIPRANGE, ],
  enum_types=[
    _WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR_SCORECONVERTER,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1222,
  serialized_end=2058,
)


_MASKRCNNBOXPREDICTOR = _descriptor.Descriptor(
  name='MaskRCNNBoxPredictor',
  full_name='rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='fc_hyperparams', full_name='rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor.fc_hyperparams', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='use_dropout', full_name='rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor.use_dropout', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dropout_keep_probability', full_name='rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor.dropout_keep_probability', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0.5),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='box_code_size', full_name='rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor.box_code_size', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=4,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='conv_hyperparams', full_name='rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor.conv_hyperparams', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='predict_instance_masks', full_name='rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor.predict_instance_masks', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mask_prediction_conv_depth', full_name='rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor.mask_prediction_conv_depth', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=256,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='predict_keypoints', full_name='rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor.predict_keypoints', index=7,
      number=8, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mask_height', full_name='rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor.mask_height', index=8,
      number=9, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=15,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mask_width', full_name='rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor.mask_width', index=9,
      number=10, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=15,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mask_prediction_num_conv_layers', full_name='rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor.mask_prediction_num_conv_layers', index=10,
      number=11, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='masks_are_class_agnostic', full_name='rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor.masks_are_class_agnostic', index=11,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='share_box_across_classes', full_name='rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor.share_box_across_classes', index=12,
      number=13, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2061,
  serialized_end=2623,
)


_RFCNBOXPREDICTOR = _descriptor.Descriptor(
  name='RfcnBoxPredictor',
  full_name='rastervision.protos.tf_object_detection.RfcnBoxPredictor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='conv_hyperparams', full_name='rastervision.protos.tf_object_detection.RfcnBoxPredictor.conv_hyperparams', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='num_spatial_bins_height', full_name='rastervision.protos.tf_object_detection.RfcnBoxPredictor.num_spatial_bins_height', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=3,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='num_spatial_bins_width', full_name='rastervision.protos.tf_object_detection.RfcnBoxPredictor.num_spatial_bins_width', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=3,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='depth', full_name='rastervision.protos.tf_object_detection.RfcnBoxPredictor.depth', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=1024,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='box_code_size', full_name='rastervision.protos.tf_object_detection.RfcnBoxPredictor.box_code_size', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=4,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='crop_height', full_name='rastervision.protos.tf_object_detection.RfcnBoxPredictor.crop_height', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=12,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='crop_width', full_name='rastervision.protos.tf_object_detection.RfcnBoxPredictor.crop_width', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=12,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2626,
  serialized_end=2891,
)

_BOXPREDICTOR.fields_by_name['convolutional_box_predictor'].message_type = _CONVOLUTIONALBOXPREDICTOR
_BOXPREDICTOR.fields_by_name['mask_rcnn_box_predictor'].message_type = _MASKRCNNBOXPREDICTOR
_BOXPREDICTOR.fields_by_name['rfcn_box_predictor'].message_type = _RFCNBOXPREDICTOR
_BOXPREDICTOR.fields_by_name['weight_shared_convolutional_box_predictor'].message_type = _WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR
_BOXPREDICTOR.oneofs_by_name['box_predictor_oneof'].fields.append(
  _BOXPREDICTOR.fields_by_name['convolutional_box_predictor'])
_BOXPREDICTOR.fields_by_name['convolutional_box_predictor'].containing_oneof = _BOXPREDICTOR.oneofs_by_name['box_predictor_oneof']
_BOXPREDICTOR.oneofs_by_name['box_predictor_oneof'].fields.append(
  _BOXPREDICTOR.fields_by_name['mask_rcnn_box_predictor'])
_BOXPREDICTOR.fields_by_name['mask_rcnn_box_predictor'].containing_oneof = _BOXPREDICTOR.oneofs_by_name['box_predictor_oneof']
_BOXPREDICTOR.oneofs_by_name['box_predictor_oneof'].fields.append(
  _BOXPREDICTOR.fields_by_name['rfcn_box_predictor'])
_BOXPREDICTOR.fields_by_name['rfcn_box_predictor'].containing_oneof = _BOXPREDICTOR.oneofs_by_name['box_predictor_oneof']
_BOXPREDICTOR.oneofs_by_name['box_predictor_oneof'].fields.append(
  _BOXPREDICTOR.fields_by_name['weight_shared_convolutional_box_predictor'])
_BOXPREDICTOR.fields_by_name['weight_shared_convolutional_box_predictor'].containing_oneof = _BOXPREDICTOR.oneofs_by_name['box_predictor_oneof']
_CONVOLUTIONALBOXPREDICTOR.fields_by_name['conv_hyperparams'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_hyperparams__pb2._HYPERPARAMS
_CONVOLUTIONALBOXPREDICTOR.fields_by_name['mask_head'].message_type = _MASKHEAD
_WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR_BOXENCODINGSCLIPRANGE.containing_type = _WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR
_WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR.fields_by_name['conv_hyperparams'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_hyperparams__pb2._HYPERPARAMS
_WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR.fields_by_name['mask_head'].message_type = _MASKHEAD
_WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR.fields_by_name['score_converter'].enum_type = _WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR_SCORECONVERTER
_WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR.fields_by_name['box_encodings_clip_range'].message_type = _WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR_BOXENCODINGSCLIPRANGE
_WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR_SCORECONVERTER.containing_type = _WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR
_MASKRCNNBOXPREDICTOR.fields_by_name['fc_hyperparams'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_hyperparams__pb2._HYPERPARAMS
_MASKRCNNBOXPREDICTOR.fields_by_name['conv_hyperparams'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_hyperparams__pb2._HYPERPARAMS
_RFCNBOXPREDICTOR.fields_by_name['conv_hyperparams'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_hyperparams__pb2._HYPERPARAMS
DESCRIPTOR.message_types_by_name['BoxPredictor'] = _BOXPREDICTOR
DESCRIPTOR.message_types_by_name['MaskHead'] = _MASKHEAD
DESCRIPTOR.message_types_by_name['ConvolutionalBoxPredictor'] = _CONVOLUTIONALBOXPREDICTOR
DESCRIPTOR.message_types_by_name['WeightSharedConvolutionalBoxPredictor'] = _WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR
DESCRIPTOR.message_types_by_name['MaskRCNNBoxPredictor'] = _MASKRCNNBOXPREDICTOR
DESCRIPTOR.message_types_by_name['RfcnBoxPredictor'] = _RFCNBOXPREDICTOR

BoxPredictor = _reflection.GeneratedProtocolMessageType('BoxPredictor', (_message.Message,), dict(
  DESCRIPTOR = _BOXPREDICTOR,
  __module__ = 'rastervision.protos.tf_object_detection.box_predictor_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.BoxPredictor)
  ))
_sym_db.RegisterMessage(BoxPredictor)

MaskHead = _reflection.GeneratedProtocolMessageType('MaskHead', (_message.Message,), dict(
  DESCRIPTOR = _MASKHEAD,
  __module__ = 'rastervision.protos.tf_object_detection.box_predictor_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.MaskHead)
  ))
_sym_db.RegisterMessage(MaskHead)

ConvolutionalBoxPredictor = _reflection.GeneratedProtocolMessageType('ConvolutionalBoxPredictor', (_message.Message,), dict(
  DESCRIPTOR = _CONVOLUTIONALBOXPREDICTOR,
  __module__ = 'rastervision.protos.tf_object_detection.box_predictor_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.ConvolutionalBoxPredictor)
  ))
_sym_db.RegisterMessage(ConvolutionalBoxPredictor)

WeightSharedConvolutionalBoxPredictor = _reflection.GeneratedProtocolMessageType('WeightSharedConvolutionalBoxPredictor', (_message.Message,), dict(

  BoxEncodingsClipRange = _reflection.GeneratedProtocolMessageType('BoxEncodingsClipRange', (_message.Message,), dict(
    DESCRIPTOR = _WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR_BOXENCODINGSCLIPRANGE,
    __module__ = 'rastervision.protos.tf_object_detection.box_predictor_pb2'
    # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor.BoxEncodingsClipRange)
    ))
  ,
  DESCRIPTOR = _WEIGHTSHAREDCONVOLUTIONALBOXPREDICTOR,
  __module__ = 'rastervision.protos.tf_object_detection.box_predictor_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.WeightSharedConvolutionalBoxPredictor)
  ))
_sym_db.RegisterMessage(WeightSharedConvolutionalBoxPredictor)
_sym_db.RegisterMessage(WeightSharedConvolutionalBoxPredictor.BoxEncodingsClipRange)

MaskRCNNBoxPredictor = _reflection.GeneratedProtocolMessageType('MaskRCNNBoxPredictor', (_message.Message,), dict(
  DESCRIPTOR = _MASKRCNNBOXPREDICTOR,
  __module__ = 'rastervision.protos.tf_object_detection.box_predictor_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.MaskRCNNBoxPredictor)
  ))
_sym_db.RegisterMessage(MaskRCNNBoxPredictor)

RfcnBoxPredictor = _reflection.GeneratedProtocolMessageType('RfcnBoxPredictor', (_message.Message,), dict(
  DESCRIPTOR = _RFCNBOXPREDICTOR,
  __module__ = 'rastervision.protos.tf_object_detection.box_predictor_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.RfcnBoxPredictor)
  ))
_sym_db.RegisterMessage(RfcnBoxPredictor)


# @@protoc_insertion_point(module_scope)



# flake8: noqa

from rastervision.data.raster_source.api import *
from rastervision.data.raster_transformer.api import *
from rastervision.data.label_source.api import *
from rastervision.data.label_store.api import *
from rastervision.data.vector_source.api import *

from rastervision.data.dataset_config import DatasetConfig
from rastervision.data.scene_config import SceneConfig



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rastervision/protos/command.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from rastervision.protos import analyzer_pb2 as rastervision_dot_protos_dot_analyzer__pb2
from rastervision.protos import augmentor_pb2 as rastervision_dot_protos_dot_augmentor__pb2
from rastervision.protos import task_pb2 as rastervision_dot_protos_dot_task__pb2
from rastervision.protos import backend_pb2 as rastervision_dot_protos_dot_backend__pb2
from rastervision.protos import scene_pb2 as rastervision_dot_protos_dot_scene__pb2
from rastervision.protos import evaluator_pb2 as rastervision_dot_protos_dot_evaluator__pb2
from rastervision.protos import plugin_pb2 as rastervision_dot_protos_dot_plugin__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='rastervision/protos/command.proto',
  package='rv.protos',
  syntax='proto2',
  serialized_pb=_b('\n!rastervision/protos/command.proto\x12\trv.protos\x1a\x1cgoogle/protobuf/struct.proto\x1a\"rastervision/protos/analyzer.proto\x1a#rastervision/protos/augmentor.proto\x1a\x1erastervision/protos/task.proto\x1a!rastervision/protos/backend.proto\x1a\x1frastervision/protos/scene.proto\x1a#rastervision/protos/evaluator.proto\x1a rastervision/protos/plugin.proto\"\x80\x0c\n\rCommandConfig\x12\x14\n\x0c\x63ommand_type\x18\x01 \x02(\t\x12\x10\n\x08root_uri\x18\x02 \x02(\t\x12@\n\x0e\x61nalyze_config\x18\x03 \x01(\x0b\x32&.rv.protos.CommandConfig.AnalyzeConfigH\x00\x12:\n\x0b\x63hip_config\x18\x04 \x01(\x0b\x32#.rv.protos.CommandConfig.ChipConfigH\x00\x12<\n\x0ctrain_config\x18\x05 \x01(\x0b\x32$.rv.protos.CommandConfig.TrainConfigH\x00\x12@\n\x0epredict_config\x18\x06 \x01(\x0b\x32&.rv.protos.CommandConfig.PredictConfigH\x00\x12:\n\x0b\x65val_config\x18\x07 \x01(\x0b\x32#.rv.protos.CommandConfig.EvalConfigH\x00\x12>\n\rbundle_config\x18\x08 \x01(\x0b\x32%.rv.protos.CommandConfig.BundleConfigH\x00\x12\x30\n\rcustom_config\x18\x0b \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x12(\n\x07plugins\x18\t \x01(\x0b\x32\x17.rv.protos.PluginConfig\x12\x13\n\x08split_id\x18\n \x01(\x05:\x01\x30\x1a\x8a\x01\n\rAnalyzeConfig\x12#\n\x04task\x18\x01 \x02(\x0b\x32\x15.rv.protos.TaskConfig\x12&\n\x06scenes\x18\x02 \x03(\x0b\x32\x16.rv.protos.SceneConfig\x12,\n\tanalyzers\x18\x03 \x03(\x0b\x32\x19.rv.protos.AnalyzerConfig\x1a\xe6\x01\n\nChipConfig\x12#\n\x04task\x18\x01 \x02(\x0b\x32\x15.rv.protos.TaskConfig\x12)\n\x07\x62\x61\x63kend\x18\x02 \x02(\x0b\x32\x18.rv.protos.BackendConfig\x12.\n\naugmentors\x18\x03 \x03(\x0b\x32\x1a.rv.protos.AugmentorConfig\x12,\n\x0ctrain_scenes\x18\x04 \x03(\x0b\x32\x16.rv.protos.SceneConfig\x12*\n\nval_scenes\x18\x05 \x03(\x0b\x32\x16.rv.protos.SceneConfig\x1a]\n\x0bTrainConfig\x12#\n\x04task\x18\x01 \x02(\x0b\x32\x15.rv.protos.TaskConfig\x12)\n\x07\x62\x61\x63kend\x18\x02 \x02(\x0b\x32\x18.rv.protos.BackendConfig\x1a\x87\x01\n\rPredictConfig\x12#\n\x04task\x18\x01 \x02(\x0b\x32\x15.rv.protos.TaskConfig\x12)\n\x07\x62\x61\x63kend\x18\x02 \x02(\x0b\x32\x18.rv.protos.BackendConfig\x12&\n\x06scenes\x18\x03 \x03(\x0b\x32\x16.rv.protos.SceneConfig\x1a\xb4\x01\n\nEvalConfig\x12#\n\x04task\x18\x01 \x02(\x0b\x32\x15.rv.protos.TaskConfig\x12)\n\x07\x62\x61\x63kend\x18\x02 \x02(\x0b\x32\x18.rv.protos.BackendConfig\x12&\n\x06scenes\x18\x03 \x03(\x0b\x32\x16.rv.protos.SceneConfig\x12.\n\nevaluators\x18\x04 \x03(\x0b\x32\x1a.rv.protos.EvaluatorConfig\x1a\xb3\x01\n\x0c\x42undleConfig\x12,\n\tanalyzers\x18\x01 \x03(\x0b\x32\x19.rv.protos.AnalyzerConfig\x12#\n\x04task\x18\x02 \x02(\x0b\x32\x15.rv.protos.TaskConfig\x12)\n\x07\x62\x61\x63kend\x18\x03 \x02(\x0b\x32\x18.rv.protos.BackendConfig\x12%\n\x05scene\x18\x04 \x02(\x0b\x32\x16.rv.protos.SceneConfigB\x10\n\x0e\x63ommand_config')
  ,
  dependencies=[google_dot_protobuf_dot_struct__pb2.DESCRIPTOR,rastervision_dot_protos_dot_analyzer__pb2.DESCRIPTOR,rastervision_dot_protos_dot_augmentor__pb2.DESCRIPTOR,rastervision_dot_protos_dot_task__pb2.DESCRIPTOR,rastervision_dot_protos_dot_backend__pb2.DESCRIPTOR,rastervision_dot_protos_dot_scene__pb2.DESCRIPTOR,rastervision_dot_protos_dot_evaluator__pb2.DESCRIPTOR,rastervision_dot_protos_dot_plugin__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_COMMANDCONFIG_ANALYZECONFIG = _descriptor.Descriptor(
  name='AnalyzeConfig',
  full_name='rv.protos.CommandConfig.AnalyzeConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='task', full_name='rv.protos.CommandConfig.AnalyzeConfig.task', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='scenes', full_name='rv.protos.CommandConfig.AnalyzeConfig.scenes', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='analyzers', full_name='rv.protos.CommandConfig.AnalyzeConfig.analyzers', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=872,
  serialized_end=1010,
)

_COMMANDCONFIG_CHIPCONFIG = _descriptor.Descriptor(
  name='ChipConfig',
  full_name='rv.protos.CommandConfig.ChipConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='task', full_name='rv.protos.CommandConfig.ChipConfig.task', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='backend', full_name='rv.protos.CommandConfig.ChipConfig.backend', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='augmentors', full_name='rv.protos.CommandConfig.ChipConfig.augmentors', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='train_scenes', full_name='rv.protos.CommandConfig.ChipConfig.train_scenes', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='val_scenes', full_name='rv.protos.CommandConfig.ChipConfig.val_scenes', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1013,
  serialized_end=1243,
)

_COMMANDCONFIG_TRAINCONFIG = _descriptor.Descriptor(
  name='TrainConfig',
  full_name='rv.protos.CommandConfig.TrainConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='task', full_name='rv.protos.CommandConfig.TrainConfig.task', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='backend', full_name='rv.protos.CommandConfig.TrainConfig.backend', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1245,
  serialized_end=1338,
)

_COMMANDCONFIG_PREDICTCONFIG = _descriptor.Descriptor(
  name='PredictConfig',
  full_name='rv.protos.CommandConfig.PredictConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='task', full_name='rv.protos.CommandConfig.PredictConfig.task', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='backend', full_name='rv.protos.CommandConfig.PredictConfig.backend', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='scenes', full_name='rv.protos.CommandConfig.PredictConfig.scenes', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1341,
  serialized_end=1476,
)

_COMMANDCONFIG_EVALCONFIG = _descriptor.Descriptor(
  name='EvalConfig',
  full_name='rv.protos.CommandConfig.EvalConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='task', full_name='rv.protos.CommandConfig.EvalConfig.task', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='backend', full_name='rv.protos.CommandConfig.EvalConfig.backend', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='scenes', full_name='rv.protos.CommandConfig.EvalConfig.scenes', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='evaluators', full_name='rv.protos.CommandConfig.EvalConfig.evaluators', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1479,
  serialized_end=1659,
)

_COMMANDCONFIG_BUNDLECONFIG = _descriptor.Descriptor(
  name='BundleConfig',
  full_name='rv.protos.CommandConfig.BundleConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='analyzers', full_name='rv.protos.CommandConfig.BundleConfig.analyzers', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='task', full_name='rv.protos.CommandConfig.BundleConfig.task', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='backend', full_name='rv.protos.CommandConfig.BundleConfig.backend', index=2,
      number=3, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='scene', full_name='rv.protos.CommandConfig.BundleConfig.scene', index=3,
      number=4, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1662,
  serialized_end=1841,
)

_COMMANDCONFIG = _descriptor.Descriptor(
  name='CommandConfig',
  full_name='rv.protos.CommandConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='command_type', full_name='rv.protos.CommandConfig.command_type', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='root_uri', full_name='rv.protos.CommandConfig.root_uri', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='analyze_config', full_name='rv.protos.CommandConfig.analyze_config', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='chip_config', full_name='rv.protos.CommandConfig.chip_config', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='train_config', full_name='rv.protos.CommandConfig.train_config', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='predict_config', full_name='rv.protos.CommandConfig.predict_config', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='eval_config', full_name='rv.protos.CommandConfig.eval_config', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='bundle_config', full_name='rv.protos.CommandConfig.bundle_config', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='custom_config', full_name='rv.protos.CommandConfig.custom_config', index=8,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='plugins', full_name='rv.protos.CommandConfig.plugins', index=9,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='split_id', full_name='rv.protos.CommandConfig.split_id', index=10,
      number=10, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_COMMANDCONFIG_ANALYZECONFIG, _COMMANDCONFIG_CHIPCONFIG, _COMMANDCONFIG_TRAINCONFIG, _COMMANDCONFIG_PREDICTCONFIG, _COMMANDCONFIG_EVALCONFIG, _COMMANDCONFIG_BUNDLECONFIG, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='command_config', full_name='rv.protos.CommandConfig.command_config',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=323,
  serialized_end=1859,
)

_COMMANDCONFIG_ANALYZECONFIG.fields_by_name['task'].message_type = rastervision_dot_protos_dot_task__pb2._TASKCONFIG
_COMMANDCONFIG_ANALYZECONFIG.fields_by_name['scenes'].message_type = rastervision_dot_protos_dot_scene__pb2._SCENECONFIG
_COMMANDCONFIG_ANALYZECONFIG.fields_by_name['analyzers'].message_type = rastervision_dot_protos_dot_analyzer__pb2._ANALYZERCONFIG
_COMMANDCONFIG_ANALYZECONFIG.containing_type = _COMMANDCONFIG
_COMMANDCONFIG_CHIPCONFIG.fields_by_name['task'].message_type = rastervision_dot_protos_dot_task__pb2._TASKCONFIG
_COMMANDCONFIG_CHIPCONFIG.fields_by_name['backend'].message_type = rastervision_dot_protos_dot_backend__pb2._BACKENDCONFIG
_COMMANDCONFIG_CHIPCONFIG.fields_by_name['augmentors'].message_type = rastervision_dot_protos_dot_augmentor__pb2._AUGMENTORCONFIG
_COMMANDCONFIG_CHIPCONFIG.fields_by_name['train_scenes'].message_type = rastervision_dot_protos_dot_scene__pb2._SCENECONFIG
_COMMANDCONFIG_CHIPCONFIG.fields_by_name['val_scenes'].message_type = rastervision_dot_protos_dot_scene__pb2._SCENECONFIG
_COMMANDCONFIG_CHIPCONFIG.containing_type = _COMMANDCONFIG
_COMMANDCONFIG_TRAINCONFIG.fields_by_name['task'].message_type = rastervision_dot_protos_dot_task__pb2._TASKCONFIG
_COMMANDCONFIG_TRAINCONFIG.fields_by_name['backend'].message_type = rastervision_dot_protos_dot_backend__pb2._BACKENDCONFIG
_COMMANDCONFIG_TRAINCONFIG.containing_type = _COMMANDCONFIG
_COMMANDCONFIG_PREDICTCONFIG.fields_by_name['task'].message_type = rastervision_dot_protos_dot_task__pb2._TASKCONFIG
_COMMANDCONFIG_PREDICTCONFIG.fields_by_name['backend'].message_type = rastervision_dot_protos_dot_backend__pb2._BACKENDCONFIG
_COMMANDCONFIG_PREDICTCONFIG.fields_by_name['scenes'].message_type = rastervision_dot_protos_dot_scene__pb2._SCENECONFIG
_COMMANDCONFIG_PREDICTCONFIG.containing_type = _COMMANDCONFIG
_COMMANDCONFIG_EVALCONFIG.fields_by_name['task'].message_type = rastervision_dot_protos_dot_task__pb2._TASKCONFIG
_COMMANDCONFIG_EVALCONFIG.fields_by_name['backend'].message_type = rastervision_dot_protos_dot_backend__pb2._BACKENDCONFIG
_COMMANDCONFIG_EVALCONFIG.fields_by_name['scenes'].message_type = rastervision_dot_protos_dot_scene__pb2._SCENECONFIG
_COMMANDCONFIG_EVALCONFIG.fields_by_name['evaluators'].message_type = rastervision_dot_protos_dot_evaluator__pb2._EVALUATORCONFIG
_COMMANDCONFIG_EVALCONFIG.containing_type = _COMMANDCONFIG
_COMMANDCONFIG_BUNDLECONFIG.fields_by_name['analyzers'].message_type = rastervision_dot_protos_dot_analyzer__pb2._ANALYZERCONFIG
_COMMANDCONFIG_BUNDLECONFIG.fields_by_name['task'].message_type = rastervision_dot_protos_dot_task__pb2._TASKCONFIG
_COMMANDCONFIG_BUNDLECONFIG.fields_by_name['backend'].message_type = rastervision_dot_protos_dot_backend__pb2._BACKENDCONFIG
_COMMANDCONFIG_BUNDLECONFIG.fields_by_name['scene'].message_type = rastervision_dot_protos_dot_scene__pb2._SCENECONFIG
_COMMANDCONFIG_BUNDLECONFIG.containing_type = _COMMANDCONFIG
_COMMANDCONFIG.fields_by_name['analyze_config'].message_type = _COMMANDCONFIG_ANALYZECONFIG
_COMMANDCONFIG.fields_by_name['chip_config'].message_type = _COMMANDCONFIG_CHIPCONFIG
_COMMANDCONFIG.fields_by_name['train_config'].message_type = _COMMANDCONFIG_TRAINCONFIG
_COMMANDCONFIG.fields_by_name['predict_config'].message_type = _COMMANDCONFIG_PREDICTCONFIG
_COMMANDCONFIG.fields_by_name['eval_config'].message_type = _COMMANDCONFIG_EVALCONFIG
_COMMANDCONFIG.fields_by_name['bundle_config'].message_type = _COMMANDCONFIG_BUNDLECONFIG
_COMMANDCONFIG.fields_by_name['custom_config'].message_type = google_dot_protobuf_dot_struct__pb2._STRUCT
_COMMANDCONFIG.fields_by_name['plugins'].message_type = rastervision_dot_protos_dot_plugin__pb2._PLUGINCONFIG
_COMMANDCONFIG.oneofs_by_name['command_config'].fields.append(
  _COMMANDCONFIG.fields_by_name['analyze_config'])
_COMMANDCONFIG.fields_by_name['analyze_config'].containing_oneof = _COMMANDCONFIG.oneofs_by_name['command_config']
_COMMANDCONFIG.oneofs_by_name['command_config'].fields.append(
  _COMMANDCONFIG.fields_by_name['chip_config'])
_COMMANDCONFIG.fields_by_name['chip_config'].containing_oneof = _COMMANDCONFIG.oneofs_by_name['command_config']
_COMMANDCONFIG.oneofs_by_name['command_config'].fields.append(
  _COMMANDCONFIG.fields_by_name['train_config'])
_COMMANDCONFIG.fields_by_name['train_config'].containing_oneof = _COMMANDCONFIG.oneofs_by_name['command_config']
_COMMANDCONFIG.oneofs_by_name['command_config'].fields.append(
  _COMMANDCONFIG.fields_by_name['predict_config'])
_COMMANDCONFIG.fields_by_name['predict_config'].containing_oneof = _COMMANDCONFIG.oneofs_by_name['command_config']
_COMMANDCONFIG.oneofs_by_name['command_config'].fields.append(
  _COMMANDCONFIG.fields_by_name['eval_config'])
_COMMANDCONFIG.fields_by_name['eval_config'].containing_oneof = _COMMANDCONFIG.oneofs_by_name['command_config']
_COMMANDCONFIG.oneofs_by_name['command_config'].fields.append(
  _COMMANDCONFIG.fields_by_name['bundle_config'])
_COMMANDCONFIG.fields_by_name['bundle_config'].containing_oneof = _COMMANDCONFIG.oneofs_by_name['command_config']
_COMMANDCONFIG.oneofs_by_name['command_config'].fields.append(
  _COMMANDCONFIG.fields_by_name['custom_config'])
_COMMANDCONFIG.fields_by_name['custom_config'].containing_oneof = _COMMANDCONFIG.oneofs_by_name['command_config']
DESCRIPTOR.message_types_by_name['CommandConfig'] = _COMMANDCONFIG

CommandConfig = _reflection.GeneratedProtocolMessageType('CommandConfig', (_message.Message,), dict(

  AnalyzeConfig = _reflection.GeneratedProtocolMessageType('AnalyzeConfig', (_message.Message,), dict(
    DESCRIPTOR = _COMMANDCONFIG_ANALYZECONFIG,
    __module__ = 'rastervision.protos.command_pb2'
    # @@protoc_insertion_point(class_scope:rv.protos.CommandConfig.AnalyzeConfig)
    ))
  ,

  ChipConfig = _reflection.GeneratedProtocolMessageType('ChipConfig', (_message.Message,), dict(
    DESCRIPTOR = _COMMANDCONFIG_CHIPCONFIG,
    __module__ = 'rastervision.protos.command_pb2'
    # @@protoc_insertion_point(class_scope:rv.protos.CommandConfig.ChipConfig)
    ))
  ,

  TrainConfig = _reflection.GeneratedProtocolMessageType('TrainConfig', (_message.Message,), dict(
    DESCRIPTOR = _COMMANDCONFIG_TRAINCONFIG,
    __module__ = 'rastervision.protos.command_pb2'
    # @@protoc_insertion_point(class_scope:rv.protos.CommandConfig.TrainConfig)
    ))
  ,

  PredictConfig = _reflection.GeneratedProtocolMessageType('PredictConfig', (_message.Message,), dict(
    DESCRIPTOR = _COMMANDCONFIG_PREDICTCONFIG,
    __module__ = 'rastervision.protos.command_pb2'
    # @@protoc_insertion_point(class_scope:rv.protos.CommandConfig.PredictConfig)
    ))
  ,

  EvalConfig = _reflection.GeneratedProtocolMessageType('EvalConfig', (_message.Message,), dict(
    DESCRIPTOR = _COMMANDCONFIG_EVALCONFIG,
    __module__ = 'rastervision.protos.command_pb2'
    # @@protoc_insertion_point(class_scope:rv.protos.CommandConfig.EvalConfig)
    ))
  ,

  BundleConfig = _reflection.GeneratedProtocolMessageType('BundleConfig', (_message.Message,), dict(
    DESCRIPTOR = _COMMANDCONFIG_BUNDLECONFIG,
    __module__ = 'rastervision.protos.command_pb2'
    # @@protoc_insertion_point(class_scope:rv.protos.CommandConfig.BundleConfig)
    ))
  ,
  DESCRIPTOR = _COMMANDCONFIG,
  __module__ = 'rastervision.protos.command_pb2'
  # @@protoc_insertion_point(class_scope:rv.protos.CommandConfig)
  ))
_sym_db.RegisterMessage(CommandConfig)
_sym_db.RegisterMessage(CommandConfig.AnalyzeConfig)
_sym_db.RegisterMessage(CommandConfig.ChipConfig)
_sym_db.RegisterMessage(CommandConfig.TrainConfig)
_sym_db.RegisterMessage(CommandConfig.PredictConfig)
_sym_db.RegisterMessage(CommandConfig.EvalConfig)
_sym_db.RegisterMessage(CommandConfig.BundleConfig)


# @@protoc_insertion_point(module_scope)



from rastervision.protos.class_item_pb2 \
    import ClassItem as ClassItemMsg


class ClassItem(object):
    """A class id and associated data."""

    def __init__(self, id: int, name: str = None, color=None):
        """Construct a new ClassItem.

        Color is picked randomly if it is a null value.

        Args:
            id: (int) class id
            name: (string) name of the class
            color: (string) Pillow color code
        """
        self.id = id
        self.name = name
        self.color = color

    def __eq__(self, other):
        if isinstance(other, ClassItem):
            return (self.id == other.id and self.name == other.name
                    and self.color == other.color)
        return False  # pragma: no cover

    def __repr__(self):  # pragma: no cover
        s = 'CLASS ITEM: [{}] {}'.format(self.id, self.name)
        if self.color:
            s += ' ({})'.format(self.color)
        return s

    def to_proto(self):
        return ClassItemMsg(id=self.id, name=self.name, color=self.color)

    @staticmethod
    def from_proto(msg):
        return ClassItem(id=msg.id, name=msg.name, color=msg.color)


class ClassMap(object):
    """A map from class_id to ClassItem.

    The class ids should be integers >= 1. For semantic segmentation,
    the class 0 is reserved for use as an "ignore" class, which denotes
    pixels that should not be used for training the model or evaluating it.
    """

    def __init__(self, class_items):
        """Construct a new ClassMap.

        Args:
            class_items: list of ClassItems
        """
        self.class_item_map = {}
        for class_item in class_items:
            self.class_item_map[class_item.id] = class_item

    def copy(self):
        return ClassMap(self.class_item_map.values())

    def add_nodata_item(self):
        if 0 not in self.get_keys():
            self.class_item_map[0] = ClassItem(0, 'nodata', 'grey')

    def get_by_id(self, id):
        """Return a ClassItem by its id.

        Args:
            id: (int) id of class
        """
        return self.class_item_map[id]

    def get_by_name(self, name):
        for item in self.get_items():
            if name == item.name:
                return item
        raise ValueError('{} is not a name in this ClassMap.'.format(name))

    def get_keys(self):
        """Return the keys."""
        return list(self.class_item_map.keys())

    def get_items(self):
        """Return list of ClassItems."""
        return list(self.class_item_map.values())

    def get_class_names(self):
        """Return list of class names sorted by id."""
        sorted_items = sorted(self.get_items(), key=lambda item: item.id)
        return [item.name for item in sorted_items]

    def __len__(self):
        return len(self.get_items())

    def has_all_colors(self):
        for item in self.get_items():
            if not item.color:
                return False
        return True

    def get_category_index(self):
        """Get the corresponding category_index used by TF Object Detection."""
        category_index = {}
        for class_item in self.get_items():
            category_index[class_item.id] = {
                'id': class_item.id,
                'name': class_item.name
            }
        return category_index

    def to_proto(self):
        """Transform a ClassMap into
        a list of ClassItem protobuf messages
        """
        return [item.to_proto() for item in self.get_items()]

    @staticmethod
    def construct_from(classes):
        """Construct ClassMap from a number of different
           representations.

            Args:
                classes: One of the following:
                         - a ClassMap
                         - a list of class names
                         - a list of ClassItem protobuf messages
                         - a list of ClassItems
                         - a dict which maps class names to class ids
                         - a dict which maps class names to a tuple of
                           (class_id, color), where color is a PIL color string.
        """
        result = None
        if type(classes) is ClassMap:
            result = classes
        elif type(classes) is dict:
            item_list = []
            if not len(classes.items()) == 0:
                if type(list(classes.items())[0][1]) is tuple:
                    # This dict already has colors mapped to class ids
                    for name, (class_id, color) in classes.items():
                        item_list.append(ClassItem(class_id, name, color))
                else:
                    # Map items to empty colors
                    for name, class_id in classes.items():
                        item_list.append(ClassItem(class_id, name))
            result = ClassMap(item_list)
        elif type(classes) is list:
            item_list = []
            if not len(classes) == 0:
                if type(classes[0]) is ClassItemMsg:
                    for item in classes:
                        item_list.append(
                            ClassItem(item.id, item.name, item.color))
                elif type(classes[0]) is str:
                    for i, name in enumerate(classes):
                        item_list.append(ClassItem(i + 1, name))
                else:
                    item_list = classes
            result = ClassMap(item_list)
        else:
            raise Exception('Cannot convert type {} to ClassMap'.format(
                type(classes)))

        return result



# flake8: noqa

from rastervision.data.label.labels import *
from rastervision.data.label.chip_classification_labels import *
from rastervision.data.label.object_detection_labels import *
from rastervision.data.label.semantic_segmentation_labels import *



from abc import ABC, abstractmethod

from shapely.geometry import shape, mapping
import shapely

from rastervision.data.vector_source.class_inference import (
    ClassInference, ClassInferenceOptions)


def transform_geojson(geojson,
                      crs_transformer,
                      line_bufs=None,
                      point_bufs=None,
                      to_map_coords=False):
    def is_empty_feat(f):
        # This was added to handle empty geoms which appear when using
        # OSM vector tiles.
        return ((not f.get('geometry'))
                or ((not f['geometry'].get('coordinates')) and
                    (not f['geometry'].get('geometries'))))

    new_features = []
    for f in geojson['features']:
        if is_empty_feat(f):
            continue

        geom = shape(f['geometry'])

        # Convert map to pixel coords. We need to convert to pixel coords before applying
        # buffering because those values are assumed to be in pixel units.
        def m2p(x, y, z=None):
            return crs_transformer.map_to_pixel((x, y))

        geom = shapely.ops.transform(m2p, geom)

        # Split GeometryCollection into list of geoms.
        geoms = [geom]
        if geom.geom_type == 'GeometryCollection':
            geoms = list(geom)

        # Split any MultiX to list of X.
        new_geoms = []
        for g in geoms:
            if g.geom_type in [
                    'MultiPolygon', 'MultiPoint', 'MultiLineString'
            ]:
                new_geoms.extend(list(g))
            else:
                new_geoms.append(g)
        geoms = new_geoms

        # Buffer geoms.
        class_id = f['properties']['class_id']
        new_geoms = []
        for g in geoms:
            if g.geom_type == 'LineString':
                line_buf = 1
                if line_bufs is not None:
                    line_buf = line_bufs.get(class_id, 1)
                # If line_buf for the class_id was explicitly set as None, then
                # don't buffer.
                if line_buf is not None:
                    g = g.buffer(line_buf)
                new_geoms.append(g)
            elif g.geom_type == 'Point':
                point_buf = 1
                if point_bufs is not None:
                    point_buf = point_bufs.get(class_id, 1)
                # If point_buf for the class_id was explicitly set as None, then
                # don't buffer.
                if point_buf is not None:
                    g = g.buffer(point_buf)
                new_geoms.append(g)
            else:
                # Use buffer trick to handle self-intersecting polygons. Buffer returns
                # a MultiPolygon if there is a bowtie, so we have to convert it to a
                # list of Polygons.
                poly_buf = g.buffer(0)
                if poly_buf.geom_type == 'MultiPolygon':
                    new_geoms.extend(list(poly_buf))
                else:
                    new_geoms.append(poly_buf)
        geoms = new_geoms

        # Convert back to map coords if desired. This is here so the QGIS plugin can
        # take the GeoJSON produced by a VectorSource and display it on a map.
        if to_map_coords:

            def p2m(x, y, z=None):
                return crs_transformer.pixel_to_map((x, y))

            geoms = [shapely.ops.transform(p2m, g) for g in geoms]

        for g in geoms:
            new_f = {
                'type': 'Feature',
                'geometry': mapping(g),
                'properties': f['properties']
            }
            # Have to check for empty features again which could have been introduced
            # when splitting apart multi-geoms.
            if not is_empty_feat(new_f):
                new_features.append(new_f)

    return {'type': 'FeatureCollection', 'features': new_features}


class VectorSource(ABC):
    """A source of vector data."""

    def __init__(self,
                 crs_transformer,
                 line_bufs=None,
                 point_bufs=None,
                 class_inf_opts=None):
        """Constructor.

        Args:
            crs_transformer: (CRSTransformer)
            line_bufs: (dict or None) If none, uses default buffer value of 1. Otherwise,
                a map from class_id to number of pixels to buffer by. If the buffer value
                is None, then no buffering will be performed and the LineString or Point
                won't get converted to a Polygon. Not converting to Polygon is
                incompatible with the currently available LabelSources, but may be useful
                in the future.
            point_bufs: (dict or None) same as above, but used for buffering Points into
                Polygons.
            class_inf_opts: (ClassInferenceOptions)
        """
        self.crs_transformer = crs_transformer
        self.line_bufs = line_bufs
        self.point_bufs = point_bufs
        if class_inf_opts is None:
            class_inf_opts = ClassInferenceOptions()
        self.class_inference = ClassInference(class_inf_opts)

        self.geojson = None

    def get_geojson(self, to_map_coords=False):
        """Return normalized GeoJSON.

        This infers a class_id property for each feature, converts to pixels coords (by
        default), removes empty features, splits apart multi-geoms and geom collections
        into single geometries, and buffers lines and points into Polygons.

        Args:
            to_map_coords: If true, will return GeoJSON in map coordinates.

        Returns:
            dict in GeoJSON format
        """
        if self.geojson is None:
            self.geojson = self._get_geojson()

        geojson = transform_geojson(
            self.geojson,
            self.crs_transformer,
            self.line_bufs,
            self.point_bufs,
            to_map_coords=to_map_coords)

        return geojson

    @abstractmethod
    def _get_geojson(self):
        """Return GeoJSON with class_ids in the properties."""
        pass



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rastervision/protos/tf_object_detection/multiscale_anchor_generator.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='rastervision/protos/tf_object_detection/multiscale_anchor_generator.proto',
  package='rastervision.protos.tf_object_detection',
  syntax='proto2',
  serialized_pb=_b('\nIrastervision/protos/tf_object_detection/multiscale_anchor_generator.proto\x12\'rastervision.protos.tf_object_detection\"\xba\x01\n\x19MultiscaleAnchorGenerator\x12\x14\n\tmin_level\x18\x01 \x01(\x05:\x01\x33\x12\x14\n\tmax_level\x18\x02 \x01(\x05:\x01\x37\x12\x17\n\x0c\x61nchor_scale\x18\x03 \x01(\x02:\x01\x34\x12\x15\n\raspect_ratios\x18\x04 \x03(\x02\x12\x1c\n\x11scales_per_octave\x18\x05 \x01(\x05:\x01\x32\x12#\n\x15normalize_coordinates\x18\x06 \x01(\x08:\x04true')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_MULTISCALEANCHORGENERATOR = _descriptor.Descriptor(
  name='MultiscaleAnchorGenerator',
  full_name='rastervision.protos.tf_object_detection.MultiscaleAnchorGenerator',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='min_level', full_name='rastervision.protos.tf_object_detection.MultiscaleAnchorGenerator.min_level', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=3,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='max_level', full_name='rastervision.protos.tf_object_detection.MultiscaleAnchorGenerator.max_level', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=7,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='anchor_scale', full_name='rastervision.protos.tf_object_detection.MultiscaleAnchorGenerator.anchor_scale', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(4),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='aspect_ratios', full_name='rastervision.protos.tf_object_detection.MultiscaleAnchorGenerator.aspect_ratios', index=3,
      number=4, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='scales_per_octave', full_name='rastervision.protos.tf_object_detection.MultiscaleAnchorGenerator.scales_per_octave', index=4,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='normalize_coordinates', full_name='rastervision.protos.tf_object_detection.MultiscaleAnchorGenerator.normalize_coordinates', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=True,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=119,
  serialized_end=305,
)

DESCRIPTOR.message_types_by_name['MultiscaleAnchorGenerator'] = _MULTISCALEANCHORGENERATOR

MultiscaleAnchorGenerator = _reflection.GeneratedProtocolMessageType('MultiscaleAnchorGenerator', (_message.Message,), dict(
  DESCRIPTOR = _MULTISCALEANCHORGENERATOR,
  __module__ = 'rastervision.protos.tf_object_detection.multiscale_anchor_generator_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.MultiscaleAnchorGenerator)
  ))
_sym_db.RegisterMessage(MultiscaleAnchorGenerator)


# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rastervision/protos/keras_classification/optimizer.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='rastervision/protos/keras_classification/optimizer.proto',
  package='keras_classification.protos',
  syntax='proto2',
  serialized_pb=_b('\n8rastervision/protos/keras_classification/optimizer.proto\x12\x1bkeras_classification.protos\"r\n\tOptimizer\x12\x39\n\x04type\x18\x04 \x02(\x0e\x32+.keras_classification.protos.Optimizer.Type\x12\x0f\n\x07init_lr\x18\x05 \x02(\x02\"\x19\n\x04Type\x12\x08\n\x04\x41\x44\x41M\x10\x01\x12\x07\n\x03SGD\x10\x02')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)



_OPTIMIZER_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='keras_classification.protos.Optimizer.Type',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ADAM', index=0, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SGD', index=1, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=178,
  serialized_end=203,
)
_sym_db.RegisterEnumDescriptor(_OPTIMIZER_TYPE)


_OPTIMIZER = _descriptor.Descriptor(
  name='Optimizer',
  full_name='keras_classification.protos.Optimizer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='keras_classification.protos.Optimizer.type', index=0,
      number=4, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='init_lr', full_name='keras_classification.protos.Optimizer.init_lr', index=1,
      number=5, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _OPTIMIZER_TYPE,
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=89,
  serialized_end=203,
)

_OPTIMIZER.fields_by_name['type'].enum_type = _OPTIMIZER_TYPE
_OPTIMIZER_TYPE.containing_type = _OPTIMIZER
DESCRIPTOR.message_types_by_name['Optimizer'] = _OPTIMIZER

Optimizer = _reflection.GeneratedProtocolMessageType('Optimizer', (_message.Message,), dict(
  DESCRIPTOR = _OPTIMIZER,
  __module__ = 'rastervision.protos.keras_classification.optimizer_pb2'
  # @@protoc_insertion_point(class_scope:keras_classification.protos.Optimizer)
  ))
_sym_db.RegisterMessage(Optimizer)


# @@protoc_insertion_point(module_scope)



import numpy as np
from shapely.strtree import STRtree
from shapely.geometry import shape

import rastervision as rv
from rastervision.data import ChipClassificationLabels
from rastervision.data.label_source import LabelSource
from rastervision.core import Box


def infer_cell(cell, str_tree, ioa_thresh, use_intersection_over_cell,
               background_class_id, pick_min_class_id):
    """Infer the class_id of a cell given a set of polygons.

    Given a cell and a set of polygons, the problem is to infer the class_id
    that best captures the content of the cell. This is non-trivial since there
    can be multiple polygons of differing classes overlapping with the cell.
    Any polygons that sufficiently overlaps with the cell are in the running for
    setting the class_id. If there are none in the running, the cell is either
    considered null or background. See args for more details.

    Args:
        cell: Box
        str_tree: (STRtree) collection of geoms in scene used for geometric queries.
            The geoms need to have class_id monkey-patched onto them.
        ioa_thresh: (float) the minimum IOA of a polygon and cell for that
            polygon to be a candidate for setting the class_id
        use_intersection_over_cell: (bool) If true, then use the area of the
            cell as the denominator in the IOA. Otherwise, use the area of the
            polygon.
        background_class_id: (None or int) If not None, class_id to use as the
            background class; ie. the one that is used when a window contains
            no boxes. If not set, empty windows have None set as their class_id
            which is considered a null value.
        pick_min_class_id: If true, the class_id for a cell is the minimum
            class_id of the boxes in that cell. Otherwise, pick the class_id of
            the box covering the greatest area.
    """
    cell_geom = cell.to_shapely()
    inter_polys = str_tree.query(cell_geom)

    inter_over_cells = []
    inter_over_polys = []
    class_ids = []

    # Find polygons whose intersection with the cell is big enough.
    for poly in inter_polys:
        inter = poly.intersection(cell_geom)
        inter_over_cell = inter.area / cell_geom.area
        inter_over_poly = inter.area / poly.area

        if use_intersection_over_cell:
            enough_inter = inter_over_cell >= ioa_thresh
        else:
            enough_inter = inter_over_poly >= ioa_thresh

        if enough_inter:
            inter_over_cells.append(inter_over_cell)
            inter_over_polys.append(inter_over_poly)
            class_ids.append(poly.class_id)

    # Infer class id for cell.
    if len(class_ids) == 0:
        class_id = (None if background_class_id == 0 else background_class_id)
    elif pick_min_class_id:
        class_id = min(class_ids)
    else:
        # Pick class_id of the polygon with the biggest intersection over
        # cell. If there is a tie, pick the first.
        class_id = class_ids[np.argmax(inter_over_cells)]

    return class_id


def infer_labels(geojson, extent, cell_size, ioa_thresh,
                 use_intersection_over_cell, pick_min_class_id,
                 background_class_id):
    """Infer ChipClassificationLabels grid from GeoJSON containing polygons.

    Given GeoJSON with polygons associated with class_ids, infer a grid of
    cells and class_ids that best captures the contents of each cell. See infer_cell for
    info on the args.

    Args:
        geojson: dict in normalized GeoJSON format (see VectorSource)
        extent: Box representing the bounds of the grid

    Returns:
        ChipClassificationLabels
    """
    labels = ChipClassificationLabels()
    cells = extent.get_windows(cell_size, cell_size)

    # We need to associate class_id with each geom. Monkey-patching it onto the geom
    # seems like a bad idea, but it's the only straightforward way of doing this
    # that I've been able to find.
    geoms = []
    for f in geojson['features']:
        g = shape(f['geometry'])
        g.class_id = f['properties']['class_id']
        geoms.append(g)
    str_tree = STRtree(geoms)

    for cell in cells:
        class_id = infer_cell(cell, str_tree, ioa_thresh,
                              use_intersection_over_cell, background_class_id,
                              pick_min_class_id)
        labels.set_cell(cell, class_id)
    return labels


def read_labels(geojson, extent=None):
    """Convert GeoJSON to ChipClassificationLabels.

    If the GeoJSON already contains a grid of cells, then it can be constructed
    in a straightforward manner without having to infer the class of cells.

    If extent is given, only labels that intersect with the extent are returned.

    Args:
        geojson: dict in normalized GeoJSON format (see VectorSource)
        extent: Box in pixel coords

    Returns:
       ChipClassificationLabels
    """
    labels = ChipClassificationLabels()

    for f in geojson['features']:
        geom = shape(f['geometry'])
        (xmin, ymin, xmax, ymax) = geom.bounds
        cell = Box(ymin, xmin, ymax, xmax)
        if extent is not None and not cell.to_shapely().intersects(
                extent.to_shapely()):
            continue

        props = f['properties']
        class_id = props['class_id']
        scores = props.get('scores')
        labels.set_cell(cell, class_id, scores)

    return labels


class ChipClassificationLabelSource(LabelSource):
    """A source of chip classification labels.

    Ideally the vector_source contains a square for each cell in the grid. But
    in reality, it can be difficult to label imagery in such an exhaustive way.
    So, this can also handle sources with non-overlapping polygons that
    do not necessarily cover the entire extent. It infers the grid of cells
    and associated class_ids using the extent and options if infer_cells is
    set to True.
    """

    def __init__(self,
                 vector_source,
                 crs_transformer,
                 class_map,
                 extent=None,
                 ioa_thresh=None,
                 use_intersection_over_cell=False,
                 pick_min_class_id=False,
                 background_class_id=None,
                 cell_size=None,
                 infer_cells=False):
        """Constructs a LabelSource for ChipClassificaiton backed by a GeoJSON file.

        Args:
            vector_source: (VectorSource or str)
            crs_transformer: CRSTransformer to convert from map coords in label
                in GeoJSON file to pixel coords.
            class_map: ClassMap used to infer class_ids from class_name
                (or label) field
            extent: Box used to filter the labels by extent or compute grid
        """
        if isinstance(vector_source, str):
            provider = rv._registry.get_vector_source_default_provider(
                vector_source)
            vector_source = provider.construct(vector_source) \
                .create_source(
                    crs_transformer=crs_transformer, extent=extent, class_map=class_map)

        geojson = vector_source.get_geojson()

        if infer_cells:
            self.labels = infer_labels(geojson, extent, cell_size, ioa_thresh,
                                       use_intersection_over_cell,
                                       pick_min_class_id, background_class_id)
        else:
            self.labels = read_labels(geojson, extent)

    def get_labels(self, window=None):
        if window is None:
            return self.labels
        return self.labels.get_singleton_labels(window)



import click

from rastervision.command import Command


class PredictCommand(Command):
    def __init__(self, command_config):
        self.command_config = command_config

    def run(self, tmp_dir=None):
        if not tmp_dir:
            tmp_dir = self.get_tmp_dir()
        msg = 'Making predictions...'

        cc = self.command_config

        backend = cc.backend.create_backend(cc.task)
        task = cc.task.create_task(backend)

        scenes = list(
            map(lambda s: s.create_scene(cc.task, tmp_dir), cc.scenes))

        click.echo(click.style(msg, fg='green'))
        task.predict(scenes, tmp_dir)



from abc import (ABC, abstractmethod)


class Analyzer(ABC):
    """Class that processes scenes to produce
       some output during the ANALYZE command, which
       can be used for later procesess.

    """

    @abstractmethod
    def process(self, scenes, tmp_dir):
        pass



from collections import defaultdict
import warnings

import click
import torch
import numpy as np

from rastervision.backend.torch_utils.object_detection.metrics import (
    compute_coco_eval, compute_class_f1)

warnings.filterwarnings('ignore')


def train_epoch(model,
                device,
                data_loader,
                opt,
                step_scheduler=None,
                epoch_scheduler=None):
    model.train()
    train_loss = defaultdict(lambda: 0.0)
    num_samples = 0

    with click.progressbar(data_loader, label='Training') as bar:
        for batch_ind, (x, y) in enumerate(bar):
            x = x.to(device)
            y = [_y.to(device) for _y in y]

            opt.zero_grad()
            loss_dict = model(x, y)
            loss_dict['total_loss'].backward()
            opt.step()
            if step_scheduler:
                step_scheduler.step()

            for k, v in loss_dict.items():
                train_loss[k] += v.item()
            num_samples += x.shape[0]

    for k, v in train_loss.items():
        train_loss[k] = v / num_samples

    return dict(train_loss)


def validate_epoch(model, device, data_loader, num_labels):
    model.eval()

    ys = []
    outs = []
    with torch.no_grad():
        with click.progressbar(data_loader, label='Validating') as bar:
            for batch_ind, (x, y) in enumerate(bar):
                x = x.to(device)
                out = model(x)

                ys.extend([_y.cpu() for _y in y])
                outs.extend([_out.cpu() for _out in out])

    coco_eval = compute_coco_eval(outs, ys, num_labels)

    metrics = {
        'map': 0.0,
        'map50': 0.0,
        'mean_f1': 0.0,
        'mean_score_thresh': 0.5
    }
    if coco_eval is not None:
        coco_metrics = coco_eval.stats
        best_f1s, best_scores = compute_class_f1(coco_eval)
        mean_f1 = np.mean(best_f1s[1:])
        mean_score_thresh = np.mean(best_scores[1:])
        metrics = {
            'map': coco_metrics[0],
            'map50': coco_metrics[1],
            'mean_f1': mean_f1,
            'mean_score_thresh': mean_score_thresh
        }
    return metrics



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rastervision/protos/tf_object_detection/faster_rcnn.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from rastervision.protos.tf_object_detection import anchor_generator_pb2 as rastervision_dot_protos_dot_tf__object__detection_dot_anchor__generator__pb2
from rastervision.protos.tf_object_detection import box_predictor_pb2 as rastervision_dot_protos_dot_tf__object__detection_dot_box__predictor__pb2
from rastervision.protos.tf_object_detection import hyperparams_pb2 as rastervision_dot_protos_dot_tf__object__detection_dot_hyperparams__pb2
from rastervision.protos.tf_object_detection import image_resizer_pb2 as rastervision_dot_protos_dot_tf__object__detection_dot_image__resizer__pb2
from rastervision.protos.tf_object_detection import losses_pb2 as rastervision_dot_protos_dot_tf__object__detection_dot_losses__pb2
from rastervision.protos.tf_object_detection import post_processing_pb2 as rastervision_dot_protos_dot_tf__object__detection_dot_post__processing__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='rastervision/protos/tf_object_detection/faster_rcnn.proto',
  package='rastervision.protos.tf_object_detection',
  syntax='proto2',
  serialized_pb=_b('\n9rastervision/protos/tf_object_detection/faster_rcnn.proto\x12\'rastervision.protos.tf_object_detection\x1a>rastervision/protos/tf_object_detection/anchor_generator.proto\x1a;rastervision/protos/tf_object_detection/box_predictor.proto\x1a\x39rastervision/protos/tf_object_detection/hyperparams.proto\x1a;rastervision/protos/tf_object_detection/image_resizer.proto\x1a\x34rastervision/protos/tf_object_detection/losses.proto\x1a=rastervision/protos/tf_object_detection/post_processing.proto\"\xc3\x0e\n\nFasterRcnn\x12\x1b\n\x10number_of_stages\x18\x01 \x01(\x05:\x01\x32\x12\x13\n\x0bnum_classes\x18\x03 \x01(\x05\x12L\n\rimage_resizer\x18\x04 \x01(\x0b\x32\x35.rastervision.protos.tf_object_detection.ImageResizer\x12^\n\x11\x66\x65\x61ture_extractor\x18\x05 \x01(\x0b\x32\x43.rastervision.protos.tf_object_detection.FasterRcnnFeatureExtractor\x12^\n\x1c\x66irst_stage_anchor_generator\x18\x06 \x01(\x0b\x32\x38.rastervision.protos.tf_object_detection.AnchorGenerator\x12\"\n\x17\x66irst_stage_atrous_rate\x18\x07 \x01(\x05:\x01\x31\x12h\n*first_stage_box_predictor_conv_hyperparams\x18\x08 \x01(\x0b\x32\x34.rastervision.protos.tf_object_detection.Hyperparams\x12\x30\n%first_stage_box_predictor_kernel_size\x18\t \x01(\x05:\x01\x33\x12,\n\x1f\x66irst_stage_box_predictor_depth\x18\n \x01(\x05:\x03\x35\x31\x32\x12\'\n\x1a\x66irst_stage_minibatch_size\x18\x0b \x01(\x05:\x03\x32\x35\x36\x12\x32\n%first_stage_positive_balance_fraction\x18\x0c \x01(\x02:\x03\x30.5\x12*\n\x1f\x66irst_stage_nms_score_threshold\x18\r \x01(\x02:\x01\x30\x12*\n\x1d\x66irst_stage_nms_iou_threshold\x18\x0e \x01(\x02:\x03\x30.7\x12&\n\x19\x66irst_stage_max_proposals\x18\x0f \x01(\x05:\x03\x33\x30\x30\x12/\n$first_stage_localization_loss_weight\x18\x10 \x01(\x02:\x01\x31\x12-\n\"first_stage_objectness_loss_weight\x18\x11 \x01(\x02:\x01\x31\x12\x19\n\x11initial_crop_size\x18\x12 \x01(\x05\x12\x1b\n\x13maxpool_kernel_size\x18\x13 \x01(\x05\x12\x16\n\x0emaxpool_stride\x18\x14 \x01(\x05\x12Y\n\x1asecond_stage_box_predictor\x18\x15 \x01(\x0b\x32\x35.rastervision.protos.tf_object_detection.BoxPredictor\x12#\n\x17second_stage_batch_size\x18\x16 \x01(\x05:\x02\x36\x34\x12+\n\x1dsecond_stage_balance_fraction\x18\x17 \x01(\x02:\x04\x30.25\x12]\n\x1csecond_stage_post_processing\x18\x18 \x01(\x0b\x32\x37.rastervision.protos.tf_object_detection.PostProcessing\x12\x30\n%second_stage_localization_loss_weight\x18\x19 \x01(\x02:\x01\x31\x12\x32\n\'second_stage_classification_loss_weight\x18\x1a \x01(\x02:\x01\x31\x12\x33\n(second_stage_mask_prediction_loss_weight\x18\x1b \x01(\x02:\x01\x31\x12U\n\x12hard_example_miner\x18\x1c \x01(\x0b\x32\x39.rastervision.protos.tf_object_detection.HardExampleMiner\x12\x65\n second_stage_classification_loss\x18\x1d \x01(\x0b\x32;.rastervision.protos.tf_object_detection.ClassificationLoss\x12\'\n\x18inplace_batchnorm_update\x18\x1e \x01(\x08:\x05\x66\x61lse\x12)\n\x1ause_matmul_crop_and_resize\x18\x1f \x01(\x08:\x05\x66\x61lse\x12$\n\x15\x63lip_anchors_to_image\x18  \x01(\x08:\x05\x66\x61lse\x12+\n\x1cuse_matmul_gather_in_matcher\x18! \x01(\x08:\x05\x66\x61lse\x12\x30\n!use_static_balanced_label_sampler\x18\" \x01(\x08:\x05\x66\x61lse\x12 \n\x11use_static_shapes\x18# \x01(\x08:\x05\x66\x61lse\x12\x1a\n\x0cresize_masks\x18$ \x01(\x08:\x04true\"x\n\x1a\x46\x61sterRcnnFeatureExtractor\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\'\n\x1b\x66irst_stage_features_stride\x18\x02 \x01(\x05:\x02\x31\x36\x12#\n\x14\x62\x61tch_norm_trainable\x18\x03 \x01(\x08:\x05\x66\x61lse')
  ,
  dependencies=[rastervision_dot_protos_dot_tf__object__detection_dot_anchor__generator__pb2.DESCRIPTOR,rastervision_dot_protos_dot_tf__object__detection_dot_box__predictor__pb2.DESCRIPTOR,rastervision_dot_protos_dot_tf__object__detection_dot_hyperparams__pb2.DESCRIPTOR,rastervision_dot_protos_dot_tf__object__detection_dot_image__resizer__pb2.DESCRIPTOR,rastervision_dot_protos_dot_tf__object__detection_dot_losses__pb2.DESCRIPTOR,rastervision_dot_protos_dot_tf__object__detection_dot_post__processing__pb2.DESCRIPTOR,])
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_FASTERRCNN = _descriptor.Descriptor(
  name='FasterRcnn',
  full_name='rastervision.protos.tf_object_detection.FasterRcnn',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='number_of_stages', full_name='rastervision.protos.tf_object_detection.FasterRcnn.number_of_stages', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=2,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='num_classes', full_name='rastervision.protos.tf_object_detection.FasterRcnn.num_classes', index=1,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='image_resizer', full_name='rastervision.protos.tf_object_detection.FasterRcnn.image_resizer', index=2,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='feature_extractor', full_name='rastervision.protos.tf_object_detection.FasterRcnn.feature_extractor', index=3,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_stage_anchor_generator', full_name='rastervision.protos.tf_object_detection.FasterRcnn.first_stage_anchor_generator', index=4,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_stage_atrous_rate', full_name='rastervision.protos.tf_object_detection.FasterRcnn.first_stage_atrous_rate', index=5,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=1,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_stage_box_predictor_conv_hyperparams', full_name='rastervision.protos.tf_object_detection.FasterRcnn.first_stage_box_predictor_conv_hyperparams', index=6,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_stage_box_predictor_kernel_size', full_name='rastervision.protos.tf_object_detection.FasterRcnn.first_stage_box_predictor_kernel_size', index=7,
      number=9, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=3,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_stage_box_predictor_depth', full_name='rastervision.protos.tf_object_detection.FasterRcnn.first_stage_box_predictor_depth', index=8,
      number=10, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=512,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_stage_minibatch_size', full_name='rastervision.protos.tf_object_detection.FasterRcnn.first_stage_minibatch_size', index=9,
      number=11, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=256,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_stage_positive_balance_fraction', full_name='rastervision.protos.tf_object_detection.FasterRcnn.first_stage_positive_balance_fraction', index=10,
      number=12, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0.5),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_stage_nms_score_threshold', full_name='rastervision.protos.tf_object_detection.FasterRcnn.first_stage_nms_score_threshold', index=11,
      number=13, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_stage_nms_iou_threshold', full_name='rastervision.protos.tf_object_detection.FasterRcnn.first_stage_nms_iou_threshold', index=12,
      number=14, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0.7),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_stage_max_proposals', full_name='rastervision.protos.tf_object_detection.FasterRcnn.first_stage_max_proposals', index=13,
      number=15, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=300,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_stage_localization_loss_weight', full_name='rastervision.protos.tf_object_detection.FasterRcnn.first_stage_localization_loss_weight', index=14,
      number=16, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(1),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_stage_objectness_loss_weight', full_name='rastervision.protos.tf_object_detection.FasterRcnn.first_stage_objectness_loss_weight', index=15,
      number=17, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(1),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='initial_crop_size', full_name='rastervision.protos.tf_object_detection.FasterRcnn.initial_crop_size', index=16,
      number=18, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='maxpool_kernel_size', full_name='rastervision.protos.tf_object_detection.FasterRcnn.maxpool_kernel_size', index=17,
      number=19, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='maxpool_stride', full_name='rastervision.protos.tf_object_detection.FasterRcnn.maxpool_stride', index=18,
      number=20, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='second_stage_box_predictor', full_name='rastervision.protos.tf_object_detection.FasterRcnn.second_stage_box_predictor', index=19,
      number=21, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='second_stage_batch_size', full_name='rastervision.protos.tf_object_detection.FasterRcnn.second_stage_batch_size', index=20,
      number=22, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=64,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='second_stage_balance_fraction', full_name='rastervision.protos.tf_object_detection.FasterRcnn.second_stage_balance_fraction', index=21,
      number=23, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(0.25),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='second_stage_post_processing', full_name='rastervision.protos.tf_object_detection.FasterRcnn.second_stage_post_processing', index=22,
      number=24, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='second_stage_localization_loss_weight', full_name='rastervision.protos.tf_object_detection.FasterRcnn.second_stage_localization_loss_weight', index=23,
      number=25, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(1),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='second_stage_classification_loss_weight', full_name='rastervision.protos.tf_object_detection.FasterRcnn.second_stage_classification_loss_weight', index=24,
      number=26, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(1),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='second_stage_mask_prediction_loss_weight', full_name='rastervision.protos.tf_object_detection.FasterRcnn.second_stage_mask_prediction_loss_weight', index=25,
      number=27, type=2, cpp_type=6, label=1,
      has_default_value=True, default_value=float(1),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='hard_example_miner', full_name='rastervision.protos.tf_object_detection.FasterRcnn.hard_example_miner', index=26,
      number=28, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='second_stage_classification_loss', full_name='rastervision.protos.tf_object_detection.FasterRcnn.second_stage_classification_loss', index=27,
      number=29, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='inplace_batchnorm_update', full_name='rastervision.protos.tf_object_detection.FasterRcnn.inplace_batchnorm_update', index=28,
      number=30, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='use_matmul_crop_and_resize', full_name='rastervision.protos.tf_object_detection.FasterRcnn.use_matmul_crop_and_resize', index=29,
      number=31, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='clip_anchors_to_image', full_name='rastervision.protos.tf_object_detection.FasterRcnn.clip_anchors_to_image', index=30,
      number=32, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='use_matmul_gather_in_matcher', full_name='rastervision.protos.tf_object_detection.FasterRcnn.use_matmul_gather_in_matcher', index=31,
      number=33, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='use_static_balanced_label_sampler', full_name='rastervision.protos.tf_object_detection.FasterRcnn.use_static_balanced_label_sampler', index=32,
      number=34, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='use_static_shapes', full_name='rastervision.protos.tf_object_detection.FasterRcnn.use_static_shapes', index=33,
      number=35, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='resize_masks', full_name='rastervision.protos.tf_object_detection.FasterRcnn.resize_masks', index=34,
      number=36, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=True,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=465,
  serialized_end=2324,
)


_FASTERRCNNFEATUREEXTRACTOR = _descriptor.Descriptor(
  name='FasterRcnnFeatureExtractor',
  full_name='rastervision.protos.tf_object_detection.FasterRcnnFeatureExtractor',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='rastervision.protos.tf_object_detection.FasterRcnnFeatureExtractor.type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='first_stage_features_stride', full_name='rastervision.protos.tf_object_detection.FasterRcnnFeatureExtractor.first_stage_features_stride', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=True, default_value=16,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='batch_norm_trainable', full_name='rastervision.protos.tf_object_detection.FasterRcnnFeatureExtractor.batch_norm_trainable', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2326,
  serialized_end=2446,
)

_FASTERRCNN.fields_by_name['image_resizer'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_image__resizer__pb2._IMAGERESIZER
_FASTERRCNN.fields_by_name['feature_extractor'].message_type = _FASTERRCNNFEATUREEXTRACTOR
_FASTERRCNN.fields_by_name['first_stage_anchor_generator'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_anchor__generator__pb2._ANCHORGENERATOR
_FASTERRCNN.fields_by_name['first_stage_box_predictor_conv_hyperparams'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_hyperparams__pb2._HYPERPARAMS
_FASTERRCNN.fields_by_name['second_stage_box_predictor'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_box__predictor__pb2._BOXPREDICTOR
_FASTERRCNN.fields_by_name['second_stage_post_processing'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_post__processing__pb2._POSTPROCESSING
_FASTERRCNN.fields_by_name['hard_example_miner'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_losses__pb2._HARDEXAMPLEMINER
_FASTERRCNN.fields_by_name['second_stage_classification_loss'].message_type = rastervision_dot_protos_dot_tf__object__detection_dot_losses__pb2._CLASSIFICATIONLOSS
DESCRIPTOR.message_types_by_name['FasterRcnn'] = _FASTERRCNN
DESCRIPTOR.message_types_by_name['FasterRcnnFeatureExtractor'] = _FASTERRCNNFEATUREEXTRACTOR

FasterRcnn = _reflection.GeneratedProtocolMessageType('FasterRcnn', (_message.Message,), dict(
  DESCRIPTOR = _FASTERRCNN,
  __module__ = 'rastervision.protos.tf_object_detection.faster_rcnn_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.FasterRcnn)
  ))
_sym_db.RegisterMessage(FasterRcnn)

FasterRcnnFeatureExtractor = _reflection.GeneratedProtocolMessageType('FasterRcnnFeatureExtractor', (_message.Message,), dict(
  DESCRIPTOR = _FASTERRCNNFEATUREEXTRACTOR,
  __module__ = 'rastervision.protos.tf_object_detection.faster_rcnn_pb2'
  # @@protoc_insertion_point(class_scope:rastervision.protos.tf_object_detection.FasterRcnnFeatureExtractor)
  ))
_sym_db.RegisterMessage(FasterRcnnFeatureExtractor)


# @@protoc_insertion_point(module_scope)



from copy import deepcopy

import rastervision as rv

from rastervision.backend.pytorch_chip_classification import (
    PyTorchChipClassification)
from rastervision.backend.simple_backend_config import (
    SimpleBackendConfig, SimpleBackendConfigBuilder)
from rastervision.backend.api import PYTORCH_CHIP_CLASSIFICATION


class TrainOptions():
    def __init__(self,
                 batch_size=None,
                 lr=None,
                 one_cycle=None,
                 num_epochs=None,
                 model_arch=None,
                 sync_interval=None,
                 debug=None,
                 log_tensorboard=None,
                 run_tensorboard=None):
        self.batch_size = batch_size
        self.lr = lr
        self.one_cycle = one_cycle
        self.num_epochs = num_epochs
        self.model_arch = model_arch
        self.sync_interval = sync_interval
        self.debug = debug
        self.log_tensorboard = log_tensorboard
        self.run_tensorboard = run_tensorboard

    def __setattr__(self, name, value):
        if name in ['batch_size', 'num_epochs', 'sync_interval']:
            value = int(value) if isinstance(value, float) else value
        super().__setattr__(name, value)


class PyTorchChipClassificationConfig(SimpleBackendConfig):
    train_opts_class = TrainOptions
    backend_type = PYTORCH_CHIP_CLASSIFICATION
    backend_class = PyTorchChipClassification


class PyTorchChipClassificationConfigBuilder(SimpleBackendConfigBuilder):
    config_class = PyTorchChipClassificationConfig

    def _applicable_tasks(self):
        return [rv.CHIP_CLASSIFICATION]

    def with_train_options(self,
                           batch_size=8,
                           lr=1e-4,
                           one_cycle=True,
                           num_epochs=1,
                           model_arch='resnet18',
                           sync_interval=1,
                           debug=False,
                           log_tensorboard=True,
                           run_tensorboard=True):
        """Set options for training models.

        Args:
            batch_size: (int) the batch size
            weight_decay: (float) the weight decay
            lr: (float) the learning rate if using a fixed LR
                (ie. one_cycle is False),
                or the maximum LR to use if one_cycle is True
            one_cycle: (bool) True if cyclic learning rate scheduler should
                be used. This
                cycles the LR once during the course of training and seems to
                result in a pretty consistent improvement. See lr for more
                details.
            num_epochs: (int) number of epochs (sweeps through training set) to
                train model for
            model_arch: (str) classification model backbone to use for UNet
                architecture. Any option in torchvision.models is valid, for
                example, resnet18.
            sync_interval: (int) sync training directory to cloud every
                sync_interval epochs.
            debug: (bool) if True, save debug chips (ie. visualizations of
                input to model during training) during training and use
                single-core for creating minibatches.
            log_tensorboard: (bool) if True, write events to Tensorboard log
                file
            run_tensorboard: (bool) if True, run a Tensorboard server at
                port 6006 that uses the logs generated by the log_tensorboard
                option
        """
        b = deepcopy(self)
        b.train_opts = TrainOptions(
            batch_size=batch_size,
            lr=lr,
            one_cycle=one_cycle,
            num_epochs=num_epochs,
            model_arch=model_arch,
            sync_interval=sync_interval,
            debug=debug,
            log_tensorboard=log_tensorboard,
            run_tensorboard=run_tensorboard)
        return b

    def with_pretrained_uri(self, pretrained_uri):
        """pretrained_uri should be uri of exported model file."""
        return super().with_pretrained_uri(pretrained_uri)



import os

import logging

from rastervision.runner import ExperimentRunner
from rastervision.utils.files import save_json_config
from rastervision.rv_config import RVConfig
import rastervision as rv

log = logging.getLogger(__name__)


class InProcessExperimentRunner(ExperimentRunner):
    """A class implementing functionality for running experiments within
    the present process.

    Note that the bulk of some stages still run in different process,
    as for example when a separate training script is run as a shell
    command.  The contrast between this behavior and the
    out-of-process runners (those derived from
    `OutOfProcessExperimentRunner`) is that in the latter, those shell
    commands are run from stage-specific processes.

    """

    def __init__(self, tmp_dir=None):
        self.tmp_dir = tmp_dir

    def _run_experiment(self, command_dag):
        """Runs all commands on this machine."""

        def run_commands(tmp_dir):
            for command_config in command_dag.get_sorted_commands():
                msg = command_config.to_proto()
                builder = rv._registry.get_command_config_builder(
                    msg.command_type)()
                command_config = builder.from_proto(msg).build()

                command_root_uri = command_config.root_uri
                command_basename = 'command-config-{}.json'.format(
                    command_config.split_id)
                command_uri = os.path.join(command_root_uri, command_basename)
                log.info('Saving command configuration to {}...'.format(
                    command_uri))
                save_json_config(command_config.to_proto(), command_uri)

                command = command_config.create_command()
                command.run(tmp_dir)

        if self.tmp_dir:
            run_commands(self.tmp_dir)
        else:
            with RVConfig.get_tmp_dir() as tmp_dir:
                run_commands(tmp_dir)



import random

import numpy as np

from rastervision.augmentor import Augmentor
from rastervision.core import (TrainingData, Box)


class NodataAugmentor(Augmentor):
    """Randomly add NoData values to negative chips.

    This is useful for training the model to negatively predict
    chips that are on boundaries or containing mostly NoData.
    """

    def __init__(self, aug_prob):
        self.aug_prob = aug_prob

    def process(self, training_data, tmp_dir):
        augmented = TrainingData()
        nodata_aug_prob = self.aug_prob

        for chip, window, labels in training_data:
            # If negative chip, with some probability, add a random black square
            # to chip.
            if len(labels) == 0 and random.uniform(0, 1) < nodata_aug_prob:
                size = round(random.uniform(0, 1) * chip.shape[0])
                square = Box(0, 0, chip.shape[0],
                             chip.shape[1]).make_random_square(size)
                chip = np.copy(chip)
                chip[square.ymin:square.ymax, square.xmin:square.xmax, :] = 0

            augmented.append(chip, window, labels)

        return augmented



from abc import (ABC, abstractmethod)
from typing import (List, Union)
import logging

import click
from google.protobuf import json_format

import rastervision as rv
from rastervision.runner import (CommandDefinition, CommandDAG)
from rastervision.cli import Verbosity

log = logging.getLogger(__name__)


class ExperimentRunner(ABC):
    def print_command(self, command_def, command_id=None, command_dag=None):
        verbosity = Verbosity.get()
        command_type = command_def.command_config.command_type
        split_id = command_def.command_config.split_id
        experiment_id = command_def.experiment_id
        click.echo(
            click.style('{}-{} '.format(command_type, split_id), bold=True),
            nl=False)
        click.echo('from {}'.format(experiment_id))

        if verbosity >= Verbosity.VERBOSE:
            click.echo('  INPUTS:')
            for input_uri in command_def.io_def.input_uris:
                click.echo('    {}'.format(input_uri))
            if not command_def.io_def.output_uris:
                click.echo('  NO OUTPUTS!')
            else:
                click.echo('  OUTPUTS:')
                for output_uri in command_def.io_def.output_uris:
                    click.echo('    {}'.format(output_uri))
        if verbosity >= Verbosity.VERY_VERBOSE:
            click.echo(click.style('  COMMAND CONFIGURATION', bold=True))
            click.echo('  ---------------------\n')
            click.echo('{}'.format(
                json_format.MessageToJson(
                    command_def.command_config.to_proto())))

        if command_dag:
            upstreams = command_dag.get_upstream_command_ids(command_id)
            if upstreams:
                for upstream_id in upstreams:
                    cdef = command_dag.get_command_definition(upstream_id)
                    msg = '  DEPENDS ON: {}-{} from {}'.format(
                        cdef.command_config.command_type,
                        cdef.command_config.split_id, cdef.experiment_id)
                    click.echo(click.style(msg, fg='cyan'))

    def run(self,
            experiments: Union[List[rv.ExperimentConfig], rv.ExperimentConfig],
            command_configs: Union[List[rv.CommandConfig],
                                   rv.CommandConfig] = None,
            commands_to_run=None,
            rerun_commands=False,
            skip_file_check=False,
            dry_run: bool = False,
            splits: int = 1):
        if not commands_to_run:
            commands_to_run = rv.all_commands()

        if not isinstance(experiments, list):
            experiments = [experiments]

        log.debug('Generating command definitions from experiments...')
        command_definitions = CommandDefinition.from_experiments(
            experiments, commands_to_run, splits)

        if command_configs:
            if not isinstance(command_configs, list):
                command_configs = [command_configs]
            log.debug('Generating command definitions from commands...')
            command_definitions.extend(
                CommandDefinition.from_command_configs(
                    command_configs, commands_to_run, splits))

        # Filter  out commands that don't have any output.
        log.debug('Filtering commands that do not have any output...')
        (command_definitions,
         no_output) = CommandDefinition.filter_no_output(command_definitions)

        # Print commands that have no output
        if dry_run:
            if no_output:
                print()
                click.echo(
                    click.style(
                        'Commands not run because they have no output:',
                        fg='yellow',
                        bold=True,
                        underline=True))
                for command in no_output:
                    self.print_command(command)
                print()

        # Check if there are any unsatisfied inputs.
        log.debug('Checking for missing input from configuration...')
        missing_inputs = CommandDefinition.get_missing_inputs(
            command_definitions)
        if missing_inputs:
            # TODO: Replace exception with logging?
            s = ''
            for exp_id in missing_inputs:
                s += 'In {}:\n\t{}\n'.format(
                    exp_id, '\t{}\n'.join(missing_inputs[exp_id]))

            raise rv.ConfigError('There were missing input URIs '
                                 'that are required, but were not '
                                 'able to be derived: \n{}'.format(s))

        # Remove duplicate commands, defining equality for a command by
        # the tuple (command_type, input_uris, output_uris)
        log.debug('Removing duplicate commands...')
        (unique_commands, skipped_duplicate_commands
         ) = CommandDefinition.remove_duplicates(command_definitions)

        if dry_run:
            if skipped_duplicate_commands:
                print()
                msg = ('Commands determined to be '
                       'duplicates based on input and output:')
                click.echo(
                    click.style(msg, fg='yellow', bold=True, underline=True))
                for command in skipped_duplicate_commands:
                    self.print_command(command)
                print()

        # Ensure that for each type of command, there are none that clobber
        # each other's output.
        log.debug("Ensuring commands do not overwrite each other's outputs...")
        clashing_commands = CommandDefinition.get_clashing_commands(
            unique_commands)

        if clashing_commands:
            clashing_msgs = []
            for (output_uri, c_defs) in clashing_commands:
                command_type = c_defs[0].command_config.command_type
                experiments = ', '.join(
                    map(lambda c: c.experiment_id or command_type, c_defs))
                clashing_msgs.append(
                    'The {} command in the following experiments '
                    'output {}, but are not equal: {}'.format(
                        command_type, output_uri, experiments))
            # TODO: Replace with logging?
            s = '\t\n'.join(clashing_msgs)

            raise rv.ConfigError('ERROR: Command outputs will'
                                 'override each other: \n{}\n'.format(s))

        log.debug('Constructing command DAG...')
        command_dag = CommandDAG(
            unique_commands, rerun_commands, skip_file_check=skip_file_check)

        # Print conflicating or alread fulfilled commands
        if dry_run:
            skipped_commands = command_dag.skipped_commands
            if skipped_commands:
                print()
                msg = 'Commands skipped because output already exists:'
                click.echo(
                    click.style(msg, fg='yellow', bold=True, underline=True))
                for command in skipped_commands:
                    self.print_command(command)
                print()

        # Save experiment configs
        experiments_by_id = dict(map(lambda e: (e.id, e), experiments))
        seen_ids = set([])
        for command_def in command_dag.get_command_definitions():
            if command_def.experiment_id is not None and \
               command_def.experiment_id not in seen_ids:
                seen_ids.add(command_def.experiment_id)
                experiment = experiments_by_id[command_def.experiment_id]
                if not dry_run:
                    log.debug(
                        'Saving experiment configuration for experiment {}'.
                        format(experiment.id))
                    experiment.fully_resolve().save_config()

        if dry_run:
            print()
            sorted_command_ids = command_dag.get_sorted_command_ids()
            if len(sorted_command_ids) == 0:
                click.echo(
                    click.style('No commands to run!', fg='red', bold=True))
                print()
            else:
                click.echo(
                    click.style(
                        'Commands to be run in this order:',
                        fg='green',
                        bold=True,
                        underline=True))
                for command_id in command_dag.get_sorted_command_ids():
                    command_def = command_dag.get_command_definition(
                        command_id)
                    self.print_command(command_def, command_id, command_dag)
                    print()
            self._dry_run(command_dag)
        else:
            log.debug('Running experiment...')
            self._run_experiment(command_dag)

    @abstractmethod
    def _run_experiment(self, command_dag):
        pass

    def _dry_run(self, command_dag):
        """Overridden by subclasses if they contribute dry run information.
        """
        pass

    @staticmethod
    def get_runner(runner_type):
        """Gets the runner associated with this runner type."""
        # Runner keys are upper cased.
        return rv._registry.get_experiment_runner(runner_type.upper())

    @staticmethod
    def list_runners():
        """Returns a list of valid runner keys."""
        return rv._registry.get_experiment_runner_keys()



from abc import (ABC, abstractmethod)


class RasterTransformer(ABC):
    """Transforms raw chips to be input to a neural network."""

    @abstractmethod
    def transform(self, chip, channel_order=None):
        """Transform a chip of a raster source.

        Args:
            chip: ndarray of shape [height, width, channels] This is assumed to already
                have the channel_order applied to it if channel_order is set. In other
                words, channels should be equal to len(channel_order).
            channel_order: list of indices of channels that were extracted from the
                raw imagery.

        Returns:
            [height, width, channels] numpy array
        """
        pass
